package org.cobbzilla.util.io.regex;

import com.fasterxml.jackson.databind.JsonNode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static org.cobbzilla.util.daemon.ZillaRuntime.empty;

@NoArgsConstructor
public class RegexLimitedReplacementFilter extends RegexReplacementFilter {

    @Getter @Setter private Pattern stopRegex; // the replacement will be done only up to this point if it is found
    private boolean stopRegexAlreadyFound = false;

    public RegexLimitedReplacementFilter(String regex, int group, String replacement, String stopRegex,
                                         int stopRegexMatchingFlags) {
        super(regex, group,replacement);
        this.stopRegex = Pattern.compile(stopRegex, stopRegexMatchingFlags);
    }

    @Override public void configure(JsonNode config) {
        super.configure(config);

        final JsonNode matchingFlags = config.get("matchingFlags");
        int flags = matchingFlags == null ? 0 : matchingFlags.intValue();

        this.stopRegex = Pattern.compile(config.get("stopRegex").textValue(), flags);
    }

    public RegexFilterResult apply(StringBuilder buffer, boolean eof) {
        if (empty(stopRegex)) return super.apply(buffer, eof);

        if (stopRegexAlreadyFound) return new RegexFilterResult(buffer, buffer.length(), 0);

        // look for stop regex in the buffer
        final Matcher stopRegexMatcher = stopRegex.matcher(buffer.toString());
        if (!stopRegexMatcher.find()) return super.apply(buffer, eof);

        final CharSequence partialInput = buffer.subSequence(0, stopRegexMatcher.start());
        final RegexFilterResult result = super.apply(new StringBuilder().append(partialInput), true);

        // put everything else after found stopRegex including it:
        result.buffer.append(buffer.subSequence(stopRegexMatcher.start(), buffer.length()));
        return result;
    }

}